<?php
require_once(MOD_PATH.'AProcesses/beans/Process.inc');

/**
 * MyTodoCustomDialog
 *
 * @author Tomasz Świenty
 * @version 0.1
 * @copyright Copyright (c) eDokumenty
 */
final class MyTodoCustomDialog extends DialogForm {


    /**
     * MyTodoCustomDialog constructor.
     * @param $name
     * @param null $caption
     * @param int $dstyle
     */
    public function __construct($name, $caption = NULL, $dstyle = BS_DIALOG) {

        parent::__construct($name, NULL, (BS_DIALOG & ~(SB_MIN | SB_MAX)));

        $this->width = '580px';
        $this->path = App::makeRelativePath(__FILE__);

        $this->HWND = $this->createHWND();

        if ($this->isOpened() AND !$this->noNeedForCreate()) {
            $this->create();
        }

    }


    /**
     * @param null $params
     * @param null $data
     * @throws CustomErrorException
     */
    public function open($params = NULL, &$data = NULL) : void {

        $pData = JSON::toArray($params);

        $prc_id = (is_array($pData) AND isset($pData['prc_id'])) ? $pData['prc_id'] : NULL;
        if (filter_var($prc_id, FILTER_VALIDATE_INT, ['options' => ['min_range' => 1]]) === FALSE) {
            throw new InvalidArgumentException('Brak lub nieprawidłowy identyfikator sprawy');
        }

        // walidacja uprawnienia read do sprawy
        $process = new Process($prc_id);
        if (!$process->checkAccess(LoggedUser::getUserID(), 'r')) {
            throw new CustomErrorException('Brak prawa do odczytu sprawy');
        }

        parent::open($params, $data);

        $this->setSData(['prc_id' => $prc_id]);
        $this->create();
        $this->setWData(['prc_id' => $prc_id]);

    }


    /**
     * @return bool
     * @throws CustomErrorException
     * @throws RequestAbortException
     * @throws WidgetException
     */
    public function save() : bool {

        if (!$this->isOpened()) {
            return FALSE;
        }

        $sData = $this->getSData();
        $wData = $this->getWData();

        $prc_id = (is_array($sData) AND isset($sData['prc_id'])) ? $sData['prc_id'] : NULL;
        if (filter_var($prc_id, FILTER_VALIDATE_INT, ['options' => ['min_range' => 1]]) === FALSE) {
            throw new InvalidArgumentException('Brak lub nieprawidłowy identyfikator sprawy');
        }

        // walidacja uprawnienia read do sprawy
        $process = new Process($prc_id);
        if (!$process->checkAccess(LoggedUser::getUserID(), 'r')) {
            throw new CustomErrorException('Brak prawa do odczytu sprawy');
        }

        if (!isset($wData['sumary']) OR bs_empty_str($wData['sumary'])) {
            throw new WidgetException($this->name.'sumary', 'Pole wymaga wypełnienia');
        }

        require_once(MODEL_PATH.'Tasks/Todo.inc');
        $todo = new Todo();
        $todo->set('prc_id', $prc_id);
        $todo->data = array_merge($todo->data, $wData);
        $todo->save(TRUE);

        // odpalenie RCP recznie
        require_once('./classes/EventsTimeSheet/EventsTimeSheetWidget.inc');
        $timesheetWidget = new EventsTimeSheetWidget($this->name.'etswee');
        $timesheetWidget->setContext($todo);
        $timesheetWidget->startEvent(FALSE, TRUE);

        // parentHWND = referencja do przycisku
        if (isset($this->parentHWND)) {
            JScript::registerOnLoad('$jq(\'#'.$this->parentHWND.'\').hide();');
        }

        $this->jsClose();

        return TRUE;

    }


    /**
     * @return bool
     */
    protected function create() : bool {

        if (!$this->isOpened()) {
            return FALSE;
        }

        $composer = new DialogComposer($this);
        $composer->addModernHeader(Translator::translate('Nowe zadanie'), 'eic eic-today');

        $this->sumary = new TextInput2($this->name.'sumary');
        $this->sumary->setLabel('Temat');
        $this->sumary->setRequired(TRUE);
        $this->sumary->addHTMLAttr('autocomplete', 'off');
        $this->sumary->width = 'calc(100% - '.(Dialog::SPACE_LEFT + Dialog::SPACE_RIGHT).'px)';

        $this->emp_id = new EmployeesExtSelect($this->name.'emp_id');
        $this->emp_id->width = '100%';
        $this->emp_id->height = (Widget::DEFAULT_WIDGET_HEIGHT * 2).'px';
        $this->emp_id->setLabel(Translator::translate('Dla kogo'));
        $this->emp_id->placeholder = Translator::translate('Kliknij aby wyszukać');
        $this->emp_id->alwaysShowSearchInputPlaceholder = TRUE;
        $this->emp_id->showListIcon = TRUE;
        $this->emp_id->multiselect = TRUE;
        $this->emp_id->filterString = '(o.lasfir ~* \'{SEARCH_TEXT}\')';
        $this->emp_id->valueField = 'usr_id';
        $this->emp_id->labelField = 'text';
        $this->emp_id->dataType = ExtSelect::DATA_TYPE_ARRAY;
        $this->emp_id->loadDataColumns = ['foto__'];

        $filter = '';
        if (UserRights::checkSysAcc('bswfms.extras.limitations.visibility_of_employees')) {
            require_once(MOD_PATH.'CalendarMod/CalendarModService.inc');
            $filter = ' AND ((orunid IN ('.UserRights::getOrgAccAsString().')
                OR (guest_ = '.SysContext::$usr_info['usr_id'].' AND (rights & '.(CalRights::EDIT | CalRights::READALL).') != 0)))';
        }

        require_once(MOD_PATH.'/Employers/OrganizationUnits/OrganizationUnitsHelper.inc');

        $visibilitySQL = OrganizationUnitsHelper::getWithRecursivePart(SysContext::$ent_id, SysContext::$usr_info['usr_id'], 'org_with_visibility');

        $this->emp_id->query = $visibilitySQL.'
        SELECT '.($filter ? 'DISTINCT o.lasfir,' : '').' o.usr_id, lasfir as text, o.foto__
        FROM orgtree_view o
        LEFT JOIN org_with_visibility dlu USING(orunid)
        '.($filter ? 'LEFT JOIN cal_access ON o.usr_id = owner_' : '').'
        WHERE {FILTER_STRING} AND o.ent_id = '.SysContext::$ent_id.' AND (ustype IS NULL OR ustype = \'REGULAR\') AND o.is_del IS NOT TRUE AND o.usr_id IS NOT NULL AND dlu.visible IS NOT FALSE '.$filter.'
        ORDER BY o.lasfir';

        $this->dscrpt = new TextArea2($this->name.'dscrpt');
        $this->dscrpt->height = '70px';
        $this->dscrpt->setLabel(Translator::translate('Opis'));

        $composer->addAfter(NULL, $this->sumary, 'sumary', Widget::P_TO_LEFT);
        $composer->addAfter($this->sumary, $this->emp_id, 'emp_id', Widget::P_TO_LEFT, $this->sumary);
        $composer->addAfter($this->emp_id, $this->dscrpt, 'dscrpt', Widget::P_TO_LEFT, $this->sumary);

        $composer->addModernButtonsPanel(['bSave', 'bCancel']);

        $this->add($composer);

        $this->height = ($composer->calculatedHeight + DialogComposer::BUTTONS_PANEL_HEIGHT + DialogComposer::DM_HEADER_HEIGHT + Dialog::SPACE_TOP + Dialog::SPACE_BOTTOM).'px';

        $this->bSave->tabIndex = $this->getNextTabIndex();
        $this->bCancel->tabIndex = $this->getNextTabIndex();

        return TRUE;

    }

}